{*******************************************************}
{                Delphi VCL Extensions                  }
{                                                       }
{         Copyright (c) 1999  Russia  S.Petersburg      }
{         Moshkevitch Mikhail (m_2013@hotmail.com)      }
{                                                       }
{*******************************************************}

unit MovedPanel;

interface

uses
  Windows, Messages, SysUtils,
  Classes, Graphics, Controls, Forms, Dialogs,
  ExtCtrls;

type

  TMovedPanelAnimateType = (
    anLeftToRight,
    anRightToLeft,
    anTopToBottom,
    anBottomToTop,
    anHorzBall,
    anVertBall,
    anRandomBall);

  TMovedPanel = class(TCustomPanel)
  private
    FOnTimer: TNotifyEvent;
    FOnChangeDirection : TNotifyEvent;
    FShangeDirectionX  : boolean;
    FShangeDirectionY  : boolean;
    procedure SetWordWrap(Value: Boolean);
    procedure SetTimerInterval(Value:integer);
    procedure SetAnimate(Value:boolean);
    procedure SetBorderColor(Value: TColor);
    procedure SetAnimateType(Value: TMovedPanelAnimateType);
    procedure SetOnTimer(Value: TNotifyEvent);
    procedure SetNewCaption(Value:string);//       Editor
    function  GetNewCaption:string;
    procedure WMEraseBkgnd(var Message: TWMEraseBkgnd); message WM_ERASEBKGND;
  protected
    FTimer            : integer;
    FTimerInterval    : integer;
    FAnimate          : boolean;
    FWordWrap         : boolean;
    FBorderColor      : TColor;
    FAnimateType      : TMovedPanelAnimateType;
    procedure DoDrawText(var Rect: TRect; Flags: Longint);
    procedure PaintCaption;
    procedure Paint; override;
    procedure CreateWnd; override;
    procedure CreateParams(var Params: TCreateParams); override;
    procedure Loaded; override;
    procedure UpdateTimer;
    procedure ChangeDirection;
    procedure WMTimer(var Msg: TMessage); message WM_TIMER;
  public
    constructor Create(AOwner: TComponent); override;
    destructor  Destroy;
    procedure Refresh;
  published
    property Alignment;
    property BevelInner;
    property BevelOuter;
    property BevelWidth;
    property BorderWidth;
    property BorderStyle;
    property Caption:string read GetNewCaption write SetNewCaption;
    property Ctl3D;
    property DragCursor;
    property DragMode;
    property Enabled;
    property FullRepaint;
    property Font;
    property Locked;
    property ParentColor;
    property ParentCtl3D;
    property ParentFont;
    property ParentShowHint;
    property PopupMenu;
    property ShowHint;
    property TabOrder;
    property TabStop;
    property Visible;
    property OnClick;
    property OnDblClick;
    property OnDragDrop;
    property OnDragOver;
    property OnEndDrag;
    property OnEnter;
    property OnExit;
    property OnMouseDown;
    property OnMouseMove;
    property OnMouseUp;
    property OnResize;
    //--- new
    property  Animate:boolean read FAnimate write SetAnimate;
    property  TimerInterval:integer read FTimerInterval write SetTimerInterval;
    property  WordWrap: boolean read FWordWrap write SetWordWrap;
    property  BorderColor: TColor read FBorderColor write SetBorderColor;
    property  AnimateType: TMovedPanelAnimateType read FAnimateType write SetAnimateType;
    property  OnTimer: TNotifyEvent read FOnTimer write SetOnTimer;
    property  OnChangeDirection: TNotifyEvent read FOnChangeDirection write FOnChangeDirection;
  end;

procedure Register;


//================================================================
implementation
uses
   DsgnIntf,
   MmrHintPropEditor;

{.$R *.DCR}

procedure Register;
begin
  RegisterPropertyEditor(
    TypeInfo(string),TMovedPanel,'Caption',TMmrHintProperty);
  RegisterComponents('Samples', [TMovedPanel]);
end;

constructor TMovedPanel.Create(AOwner: TComponent);
begin
  inherited;
  FBorderColor     := clBtnFace;
  FWordWrap        := false;
  FTimer           := 1;
  FTimerInterval   := 100;
  FAnimate         := false;
  FAnimateType     := anBottomToTop;
end;

destructor TMovedPanel.Destroy;
begin
  SetTimerInterval(0);
  inherited Destroy;
end;

procedure TMovedPanel.CreateWnd;
begin
  inherited CreateWnd;
  if HandleAllocated then UpdateTimer;
end;

procedure TMovedPanel.Loaded;
begin
  inherited Loaded;
end;

procedure TMovedPanel.WMEraseBkgnd(var Message: TWMEraseBkgnd);
begin
  if (csDesigning in ComponentState) then inherited
  else Message.Result:=1
end;

procedure TMovedPanel.CreateParams(var Params: TCreateParams);
begin
  inherited CreateParams(Params);
  with Params do
  begin
    ExStyle:=ExStyle or WS_EX_TRANSPARENT;  //
    { and           
      not WS_EX_WINDOWEDGE and not WS_EX_STATICEDGE and // 
      not WS_EX_DLGMODALFRAME and not WS_EX_CLIENTEDGE;
    }  
  end;
end;

procedure TMovedPanel.SetNewCaption(Value:string);
begin
  inherited Caption:=Value;
end;

function  TMovedPanel.GetNewCaption:string;
begin
  result:=inherited Caption;
end;

procedure TMovedPanel.SetWordWrap(Value: Boolean);
begin
  if FWordWrap <> Value then
  begin
    FWordWrap := Value;
    Invalidate;
  end;
end;

procedure TMovedPanel.SetBorderColor(Value: TColor);
begin
  if FBorderColor <> Value then
  begin
    FBorderColor := Value;
    Invalidate;
  end;
end;

procedure TMovedPanel.SetAnimateType(Value: TMovedPanelAnimateType);
begin
  if FAnimateType <> Value then
  begin
    FAnimateType := Value;
  end;
end;

procedure TMovedPanel.SetOnTimer(Value: TNotifyEvent);
begin
  FOnTimer := Value;
  UpdateTimer;
end;

//---------------------------  Paint ----------------------

procedure TMovedPanel.DoDrawText(var Rect: TRect; Flags: Longint);
var
  Text: string;
begin
  Text := Caption;
  if (Flags and DT_CALCRECT <> 0) and ((Text = '') and
    (Text[1] = '&') and (Text[2] = #0)) then Text := Text + ' ';
  //if not FShowAccelChar then Flags := Flags or DT_NOPREFIX;
  Canvas.Font := Font;
  if not Enabled then
  begin
    OffsetRect(Rect, 1, 1);
    Canvas.Font.Color := clBtnHighlight;
    DrawText(Canvas.Handle, PChar(Text), Length(Text), Rect, Flags);
    OffsetRect(Rect, -1, -1);
    Canvas.Font.Color := clBtnShadow;
    DrawText(Canvas.Handle, PChar(Text), Length(Text), Rect, Flags);
  end
  else
    DrawText(Canvas.Handle, PChar(Text), Length(Text), Rect, Flags);
end;

procedure TMovedPanel.PaintCaption;
const
  Alignments: array[TAlignment] of Word = (DT_LEFT, DT_RIGHT, DT_CENTER);
  WordWraps: array[Boolean] of Word = (0, DT_WORDBREAK);
var
  Rect, CalcRect: TRect;
  DrawStyle: Longint;
  i:integer;
begin
  with Canvas do
  begin
    Brush.Style := bsClear;
    i:=1+BorderWidth+BevelWidth;
    Rect := classes.Rect(i,i,
      ClientWidth-i,
      ClientHeight-i);
    { DoDrawText takes care of BiDi alignments }
    DrawStyle := DT_EXPANDTABS or WordWraps[FWordWrap] or Alignments[Alignment];
    { Calculate vertical layout }
    { if FLayout <> tlTop then }
    { //   
    begin
      CalcRect := Rect;
      DoDrawText(CalcRect, DrawStyle or DT_CALCRECT);
      OffsetRect(Rect, 0, (Height - CalcRect.Bottom) div 2);
    end;
    }
    DoDrawText(Rect, DrawStyle);
  end;
end;

procedure TMovedPanel.Paint;
const
  Alignments: array[TAlignment] of Longint = (DT_LEFT, DT_RIGHT, DT_CENTER);
var
  Rect: TRect;
  TopColor, BottomColor, cl: TColor;
  FontHeight: Integer;
  Flags: Longint;
  ps:TPenStyle;

  procedure AdjustColors(Bevel: TPanelBevel);
  begin
    TopColor := clBtnHighlight;
    if Bevel = bvLowered then TopColor := clBtnShadow;
    BottomColor := clBtnShadow;
    if Bevel = bvLowered then BottomColor := clBtnHighlight;
  end;

begin
  Rect := GetClientRect;
  if BevelOuter <> bvNone then
  begin
    AdjustColors(BevelOuter);
    Frame3D(Canvas, Rect, TopColor, BottomColor, BevelWidth);
  end;
  Frame3D(Canvas, Rect, FBorderColor, FBorderColor, BorderWidth);
  if BevelInner <> bvNone then
  begin
    AdjustColors(BevelInner);
    Frame3D(Canvas, Rect, TopColor, BottomColor, BevelWidth);
  end;
  if (csDesigning in ComponentState) then
  with Canvas do
  begin
    if Parent is TWinControl then
      cl:=TWinControl(Parent).Brush.Color
    else cl:=clBtnFace;
    Brush.Color := cl;
    Brush.Style := bsSolid;
    Pen.Color:=clBlack;
    ps:=Pen.Style;
    Pen.Style:=psDash;
    with Rect do Rectangle(Left,Top,Right,Bottom);
    Pen.Style:=ps;
  end;
  if Caption<>'' then  PaintCaption;
end;

procedure TMovedPanel.Refresh;
var cl:TColor;
begin
  with Canvas do
  begin
    if Parent is TWinControl then
      cl:=TWinControl(Parent).Brush.Color
    else cl:=clBtnFace;
    Brush.Color := cl;
    Brush.Style := bsSolid;
    FillRect(GetClientRect);
  end;
  inherited Refresh;
end;


//---------------------------  Timer ----------------------

procedure TMovedPanel.UpdateTimer;
begin
  try
    KillTimer(WindowHandle,FTimer);
    FTimer:=0;
  except
  end;
  if not FAnimate then exit;
  FTimer:=1;
  SetTimer(WindowHandle,FTimer,FTimerInterval,nil);
end;

procedure TMovedPanel.SetTimerInterval(Value:integer);
begin
  if (FTimerInterval=Value) then exit;
  FTimerInterval:=Value;
  UpdateTimer;
end;

procedure TMovedPanel.SetAnimate(Value:boolean);
begin
  if FAnimate=Value then exit;
  FAnimate:=Value;
  UpdateTimer;
end;

procedure TMovedPanel.ChangeDirection;
begin
  if Assigned(FOnChangeDirection) then FOnChangeDirection(Self);
end;

procedure TMovedPanel.WMTimer(var Msg: TMessage);
var i,j:integer;
  b:boolean;
begin
  b:=false;
  if Msg.WParam<>FTimer then exit;
  if csDesigning in ComponentState then exit;
  if Assigned(FOnTimer) then FOnTimer(Self);
  i:=Left;
  j:=Top;
  case FAnimateType of
    anLeftToRight:
      begin
        Inc(i,1);
        b:=i>Parent.Width;
        if b then i:=-Width;
      end;
    anRightToLeft:
      begin
        Dec(i,1);
        b:=i<-Width;
        if b then i:=Parent.Width;
      end;
    anTopToBottom:
      begin
        Inc(j,1);
        b:=j>Parent.Height;
        if b then j:=-Height;
      end;
    anBottomToTop:
      begin
        Dec(j,1);
        b:=j<-Height;
        if b then j:=Parent.Height;
      end;
    anHorzBall:
      begin
        if FShangeDirectionX then Inc(i,1) else  Dec(i,1);
        b:=(i>(Parent.Width-Width))or(i<0);
        if b then FShangeDirectionX:=not FShangeDirectionX;
      end;
    anVertBall:
      begin
        if FShangeDirectionY then Inc(j,1) else  Dec(j,1);
        b:=(j>(Parent.Height-Height))or(j<0);
        if b then FShangeDirectionY:=not FShangeDirectionY;
      end;
    anRandomBall:
      begin
        if FShangeDirectionX then Inc(i,1) else  Dec(i,1);
        b:=(i>(Parent.Width-Width))or(i<0);
        if b then FShangeDirectionX:=not FShangeDirectionX;
        if FShangeDirectionY then Inc(j,1) else  Dec(j,1);
        if (j>(Parent.Height-Height))or(j<0) then
        begin
          b:=true;
          FShangeDirectionY:=not FShangeDirectionY;
        end;
      end;
  end;
  SetBounds(i,j,Width,Height);
  if b then ChangeDirection;
end;

end.
